#include <itpp/itbase.h>
#include "merger.h"
#include "arx.h"

vec merger::lognorm_merge ( mat &lW ) {
	int nu=lW.rows();
	vec mu = sum ( lW ) /nu; //mean of logs
	vec lam = sum ( pow ( lW,2 ) )-nu*pow ( mu,2 );
	double coef=0.0;
	switch ( nu ) {
		case 2:
			coef=sqrt ( beta*2 ) * ( 1-0.5*sqrt ( ( 4*beta-3 ) /beta ) );
			return exp ( coef*sqrt ( lam ) + mu );
			break;
		case 3://Ration of Bessel
			break;
		case 4:
			break;
	}
	return vec ( 0 );
}

void merger::merge ( const epdf* g0 ) {
	it_file dbg ( "merger_debug.it" );

	it_assert_debug ( rv.equal ( g0->_rv() ),"Incompatible g0" );
	//Empirical density - samples
	eEmp eSmp ( rv,Ns );
	eSmp.set_parameters ( ones ( Ns ), g0 );
	Array<vec> &Smp = eSmp._samples(); //aux
	vec &w = eSmp._w(); //aux

	mat Smp_ex =ones ( rv.count() +1,Ns ); // Extended samples The last row is ones for the ARX model
	for ( int i=0;i<Ns;i++ ) {	set_col_part ( Smp_ex,i,Smp ( i ) );}

	dbg << Name ( "Smp0" ) << Smp_ex;

	// Stuff for merging
	vec lw_src ( Ns );
	vec lw_mix ( Ns );
	mat lW=zeros ( n,Ns );
	vec vec0 ( 0 );

	// Initial component in the mixture model
	mat V0=1e-8*eye ( rv.count() +1 );
	ARX A0 ( RV ( "{th_r  }", vec_1 ( rv.count() * ( rv.count() +1 ) ) ),\
	         V0, 4.0 ); //initial guess of Mix: zero mean, large variance



	// ============= MAIN LOOP ==================
	bool converged=false;
	int niter = 0;
	char str[100];
	
	vec Mix_pdf(Ns);
	while ( !converged ) {
		//Re-estimate Mix
		//Re-Initialize Mixture model
		Mix.init ( &A0, Smp_ex, Nc );
		Mix.bayesB ( Smp_ex );

		// Generate new samples
		eSmp.set_samples ( Mix.predictor ( rv ) );
		for ( int i=0;i<Ns;i++ ) {	set_col_part ( Smp_ex,i,Smp ( i ) );}

		sprintf ( str,"Mpdf%d",niter );
		for (int i=0;i<Ns;i++){Mix_pdf(i) = Mix.logpred(Smp_ex.get_col(i));}
		dbg << Name ( str ) << Mix_pdf;

		sprintf ( str,"Smp%d",niter );
		dbg << Name ( str ) << Smp_ex;

		//Importace weighting
		for ( int i=0;i<n;i++ ) {
			//Split according to dependency in rvs
			//======== Same RVs ===========
			if ( rvsinrv ( i ).length() ==rv.count() ) {
				// no need for conditioning or marginalization
				for ( int j=0;j<Ns;j++ ) {
					lw_src ( j ) =mpdfs ( i )->_epdf().evalpdflog ( Smp ( j ) );
				}
			}
			lW.set_row ( i, lw_src ); // do not divide by mix
		}
		//Importance of the mixture
		for ( int j=0;j<Ns;j++ ) {
			lw_mix ( j ) =Mix.logpred ( Smp_ex.get_col ( j ) );
		}
		sprintf ( str,"lW%d",niter );
		dbg << Name ( str ) << lW;

		w = lognorm_merge ( lW ); //merge

		sprintf ( str,"w%d",niter );
		dbg << Name ( str ) << w;

		//Importance weighting
		w /=exp ( lw_mix ); // hoping that it is not numerically sensitive...
		//renormalize
		w /=sum ( w );

		eSmp.resample(); // So that it can be used in bayes
		for ( int i=0;i<Ns;i++ ) {	set_col_part ( Smp_ex,i,Smp ( i ) );}

		sprintf ( str,"Smp_res%d",niter );
		dbg << Name ( str ) << Smp;

		// ==== stopping rule ===
		niter++;
		converged = ( niter>3 );
	}

}

